/*
** Haaf's Game Engine 1.8
** Copyright (C) 2003-2007, Relish Games
** hge.relishgames.com
**
** hge_tut03 - Using helper classes
*/


#include <cmath>
#include <ctime>
#include <vector>

#include "hge.h"
#include "hgesprite.h"
#include "hgefont.h"

// Pointer to the HGE interface.
// Helper classes require this to work.
HGE *hge=0;

HTEXTURE			texGhost;
hgeSprite*			sprGhost;
HTEXTURE			texGhost2;
hgeSprite*			sprGhost2;
HTEXTURE			texPacMan;
hgeSprite*			sprPacMan;
HTEXTURE			texFood;
hgeSprite*			sprFood;

// Pointers to the HGE objects we will use
// Handles for HGE resourcces
hgeFont*			fnt;
HEFFECT				sndEnergy;
HEFFECT				sndPacEaten;
HEFFECT				sndFoodEaten;
HEFFECT				sndDead;

HMUSIC				musGame;

// Some "gameplay" variables
float g_BoardSizeX= 512.0f;
float g_BoardSizeY= 384.0f;
float g_GhostSize= 16.0f;
float g_PacSize= 8.0f;
float g_FoodSize= 2.0f;

bool g_GameOver= false;
float g_fGameTime= 0.0f;
int	g_iScore= 0;
int g_iPlayerCount= 1;

//helperfunktionen
void Clamp( float& f, float fMin, float fMax )
{
	if( f < fMin )
	{
		f= fMin;
	}
	if( f > fMax )
	{
		f= fMax;
	}
}

void playFX( HEFFECT hFX, float fX= 0.0f, float fPitch= 1.0f ) 
{
	float pan= fX / g_BoardSizeX * 100.0f;
	Clamp( pan, -100.0f, 100.0f );
	hge->Effect_PlayEx(hFX,100,(int)pan, fPitch);
}

class Player
{
public:
	Player()
	{
		Reset();
	}

	void Reset()
	{
		m_fPosX= 0.0f;
		m_fPosY= 0.0f;
		m_fSpeedX= 0.0f;
		m_fSpeedY= 0.0f;

		m_fVulnerable= 1.0f;

		m_bFocus= false;

		m_fEnergySoundTimer= 0.0f;
	}
	
	void SetPos( float fPosX, float fPosY )
	{
		m_fPosX= fPosX;
		m_fPosY= fPosY;
	}

	float GetX()
	{
		return m_fPosX;
	}
	float GetY()
	{
		return m_fPosY;
	}

	float GetFoodPower()
	{
		if( m_bFocus )
		{
			return 500.0f + m_fVulnerable * 20.0f;
		}
		else
		{
			return 48.0f;
		}
	}

	void draw()
	{
		float fEner= fabs( m_fVulnerable * 255.0f/ 15.0f );
		if( m_fVulnerable > 0.0f )
		{
			fEner*= 5.0f;
		}
		Clamp( fEner, 0.0f, 255.0f );
		sprGhost2->SetColor( (int)(fEner) << 24 | ( m_fVulnerable > 0.0f ? 0x8040 : 0xa05000) );
		sprGhost2->RenderEx( 
			m_fPosX + g_BoardSizeX, 
			m_fPosY + g_BoardSizeY,
			0.0f,
			1.0f );

		float Factor= -m_fVulnerable * 255.0f / 3.0f;
		Clamp( Factor, 0.0f, 255.0f );
		if( m_bFocus )
		{
			Factor= 0.0f;
		}
		DWORD dwColor = (int)(Factor) << 16 | (int)(255-Factor);
		sprGhost->SetColor( 0xFF000000 | dwColor );
		sprGhost->RenderEx( 
			m_fPosX + g_BoardSizeX, 
			m_fPosY + g_BoardSizeY,
			0.0f,
			g_GhostSize / 16.0f);
	}

	void calculate( float fTimeStep )
	{
		float fAcc= 600.0f * fTimeStep;
		float fFriction= pow( 0.0625f, fTimeStep );

		m_bFocus= false;
		if (hge->Input_GetKeyState(m_iKeyFocus))
		{
			fAcc*= 0.25f;
			m_bFocus= true;
		}


		if (hge->Input_GetKeyState(m_iKeyLeft))
		{	
			m_fSpeedX-= fAcc;
		}
		if (hge->Input_GetKeyState(m_iKeyRight))
		{	
			m_fSpeedX+= fAcc;
		}
		if (hge->Input_GetKeyState(m_iKeyUp))
		{	
			m_fSpeedY-= fAcc;
		}
		if (hge->Input_GetKeyState(m_iKeyDown))
		{	
			m_fSpeedY+= fAcc;
		}

		// Do some movement calculations and collision detection	
		m_fSpeedX*= fFriction;
		m_fSpeedY*= fFriction;
		m_fPosX+= m_fSpeedX * fTimeStep;
		m_fPosY+= m_fSpeedY * fTimeStep;

		if( m_fPosX < -g_BoardSizeX + g_GhostSize || m_fPosX > g_BoardSizeX - g_GhostSize )
		{
			m_fSpeedX*= -1.0f;
		}
		if( m_fPosY < -g_BoardSizeY + g_GhostSize || m_fPosY > g_BoardSizeY - g_GhostSize )
		{
			m_fSpeedY*= -1.0f;
		}
		Clamp( m_fPosX, -g_BoardSizeX + g_GhostSize, g_BoardSizeX - g_GhostSize );
		Clamp( m_fPosY, -g_BoardSizeY + g_GhostSize, g_BoardSizeY - g_GhostSize );

		m_fVulnerable+= fTimeStep;
		Clamp( m_fVulnerable, -15.0f, 30.0f );
		m_fEnergySoundTimer-= fTimeStep;
		if( m_fEnergySoundTimer < 0 && m_bFocus && m_fVulnerable < -0.5f )
		{
			m_fEnergySoundTimer= 0.8f + m_fVulnerable / 40.0f;
			playFX( sndEnergy, GetX(), 0.5f - m_fVulnerable / 8.0f );
		}
	}

	bool IsVulnerable()
	{
		return m_fVulnerable > 0.0f || m_bFocus;
	}

	void FoodTouch( float fTimeStep )
	{
		m_fVulnerable-= fTimeStep / 5.0f;
	}

	void EatPacMan()
	{
		m_fVulnerable-= 0.25f;
	}

	void SetKeys(
		int iKeyFocus,
		int iKeyUp,
		int iKeyLeft,
		int iKeyDown,
		int iKeyRight )
	{
		m_iKeyFocus= iKeyFocus;
		m_iKeyUp= iKeyUp;
		m_iKeyLeft= iKeyLeft;
		m_iKeyDown= iKeyDown;
		m_iKeyRight= iKeyRight;
	}

private:
	float m_fPosX;
	float m_fPosY;
	float m_fSpeedX;
	float m_fSpeedY;

	float m_fVulnerable;

	bool m_bFocus;

	float m_fEnergySoundTimer;

	int m_iKeyFocus;
	int m_iKeyUp;
	int m_iKeyLeft;
	int m_iKeyDown;
	int m_iKeyRight;
};

float GetExtraCloneOffset();

class PacMan
{
public:
	PacMan()
	{
		Reset();
	}

	void Reset()
	{
		m_fPosX= 0.0f;
		m_fPosY= 0.0f;
		m_fTargetX= 0.0f;
		m_fTargetY= 0.0f;
		m_fAngle= hge->Random_Float(-3.14f, 3.14f);
		NewCloneTime();
	}

	void SetPos( float fPosX, float fPosY )
	{
		m_fPosX= fPosX;
		m_fPosY= fPosY;
		FindNewTarget();
	}

	float GetX()
	{
		return m_fPosX;
	}
	float GetY()
	{
		return m_fPosY;
	}

	void clone( PacMan& p )
	{
		m_fAngle= p.m_fAngle;
		SetPos( p.m_fPosX, p.m_fPosY );
	}

	void draw()
	{
		sprPacMan->RenderEx( 
			m_fPosX + g_BoardSizeX, 
			m_fPosY + g_BoardSizeY,
			m_fAngle,
			g_PacSize / 8.0f);
	}

	void calculate( float fTimeStep )
	{
		float Speed= 75.0f * fTimeStep;

		float fDistX= m_fPosX - m_fTargetX;
		float fDistY= m_fPosY - m_fTargetY;
		float fDist= sqrt( fDistX * fDistX + fDistY * fDistY );

		if( fDist < Speed * 5.0f )
		{
			FindNewTarget();
		}
		else
		{
			m_fPosX-= fDistX / fDist * Speed;
			m_fPosY-= fDistY / fDist * Speed;
		}
		m_fCloneTime-= fTimeStep;
	}

	void canclone( std::vector< PacMan >& PacManList )
	{
		if( m_fCloneTime < 0 )
		{
			PacMan p;
			p.clone( *this );
			PacManList.push_back( p );
			NewCloneTime();
		}
	}

	void eat()
	{
		m_fCloneTime-= 10.0f;
	}

private:
	void FindNewTarget()
	{
		float fDist= hge->Random_Float(40.0f, 100.0f);
		m_fAngle+= hge->Random_Float(-1.57f, 1.57f);
		m_fTargetX= m_fPosX + cos( m_fAngle ) * fDist;
		m_fTargetY= m_fPosY + sin( m_fAngle ) * fDist;
		Clamp( m_fTargetX, -g_BoardSizeX + g_PacSize, g_BoardSizeX - g_PacSize );
		Clamp( m_fTargetY, -g_BoardSizeY + g_PacSize, g_BoardSizeY - g_PacSize );
	}

	void NewCloneTime()
	{
		m_fCloneTime= hge->Random_Float(30.0f, 60.0f) + GetExtraCloneOffset();
	}

	float m_fPosX;
	float m_fPosY;
	float m_fTargetX;
	float m_fTargetY;
	float m_fAngle;
	float m_fCloneTime;
};

class Food
{
public:
	Food()
	{
		Reset();
	}

	void Reset()
	{
		m_fPosX= 0.0f;
		m_fPosY= 0.0f;
		m_fSpeedX= 0.0f;
		m_fSpeedY= 0.0f;
	}

	void SetPos( float fPosX, float fPosY )
	{
		m_fPosX= fPosX;
		m_fPosY= fPosY;
	}

	float GetX()
	{
		return m_fPosX;
	}
	float GetY()
	{
		return m_fPosY;
	}

	float GetNextX()
	{
		return m_fPosX + m_fSpeedX * 0.0625f;
	}
	float GetNextY()
	{
		return m_fPosY + m_fSpeedY * 0.0625f;
	}

	void draw()
	{
		sprFood->RenderEx( 
			m_fPosX + g_BoardSizeX, 
			m_fPosY + g_BoardSizeY,
			0.0f,
			g_FoodSize / 2.0f);
	}

	void calculate( float fTimeStep, std::vector< Food >& allFood, Player* p )
	{
		float fMaxDist= 30.0f;
		
		float fNearest= fMaxDist * fMaxDist;
		float fNearestX= fMaxDist;
		float fNearestY= fMaxDist;
		
		float fCenterX= 0.0f;
		float fCenterY= 0.0f;
		int NearCount= 0;

		for( int i= 0; i < allFood.size(); ++i )
		{
			float fDistX= allFood[ i ].GetX() - GetX();
			float fDistY= allFood[ i ].GetY() - GetY();
			float fDist= fDistX * fDistX + fDistY * fDistY;
			if( fDist <= fMaxDist * fMaxDist )
			{
				if( &(allFood[ i ]) == this )
				{
					continue;
				}
				fCenterX+= allFood[ i ].GetNextX();
				fCenterY+= allFood[ i ].GetNextY();
				NearCount++;

				if( fNearest > fDist )
				{
					fNearest= fDist;
					fNearestX= allFood[ i ].GetX();
					fNearestY= allFood[ i ].GetY();
				}
			}
		}
		
		float fAcc= 200.0f * fTimeStep;
		float fAccX= 0.0f;
		float fAccY= 0.0f;
		
		if( NearCount > 0 )
		{
			fCenterX/= (float)NearCount;
			fCenterY/= (float)NearCount;
			fCenterX-= m_fPosX;
			fCenterY-= m_fPosY;

			fAccX+= fCenterX * 0.125f;
			fAccY+= fCenterY * 0.125f;
		}
		
		{
			fNearestX= m_fPosX - fNearestX;
			fNearestY= m_fPosY - fNearestY;
			float fFac= 1.0f - sqrt( fNearest ) / fMaxDist;
			fAccX+= fNearestX / sqrt( fNearest ) * fFac * 5.0f;
			fAccY+= fNearestY / sqrt( fNearest ) * fFac * 5.0f;
		}

		for( int j= 0; j < g_iPlayerCount; ++j )
		{
			float fMaxSteerDist= p[ j ].GetFoodPower();
			float fSteerX= p[ j ].GetX() - m_fPosX;
			float fSteerY= p[ j ].GetY() - m_fPosY;
			float fSteerDist= sqrt( fSteerX * fSteerX + fSteerY * fSteerY );
			if( fSteerDist < g_GhostSize * 2.5f )
			{
				p[ j ].FoodTouch( fTimeStep );
			}
			if( fSteerDist < fMaxSteerDist )
			{
				float fFac= 1.0f - fSteerDist / fMaxSteerDist;
				fAccX+= fSteerX / fSteerDist * fFac;
				fAccY+= fSteerY / fSteerDist * fFac;
			}
		}
		
		{
			float fBorderSpeed= 16.0f;
			float fBorderDist= 8.0f * g_FoodSize;

			if( GetNextX() > g_BoardSizeX - fBorderDist )
			{
				fAccX-= fBorderSpeed;
			}
			if( GetNextX() < -g_BoardSizeX + fBorderDist )
			{
				fAccX+= fBorderSpeed;
			}
			if( GetNextY() > g_BoardSizeY - fBorderDist )
			{
				fAccY-= fBorderSpeed;
			}
			if( GetNextY() < -g_BoardSizeY + fBorderDist )
			{
				fAccY+= fBorderSpeed;
			}
		}

		{
			float fLength= sqrt( fAccX * fAccX + fAccY * fAccY );
			if( fLength > 0.05f )
			{
				fLength= max( 1.0f, fLength );
				m_fSpeedX+= fAcc * fAccX / fLength;
				m_fSpeedY+= fAcc * fAccY / fLength;
			}
		}


		float MaxSpeed= 100.0f;
		float fFriction= pow( 48.0f / 64.0f, fTimeStep );

		m_fSpeedX*= fFriction;
		m_fSpeedY*= fFriction;

		float fSpeed= sqrt( m_fSpeedX * m_fSpeedX + m_fSpeedY * m_fSpeedY );
		if( fSpeed > MaxSpeed )
		{
			m_fSpeedX*= MaxSpeed / fSpeed;
			m_fSpeedY*= MaxSpeed / fSpeed;
		}

		m_fPosX+= m_fSpeedX * fTimeStep;
		m_fPosY+= m_fSpeedY * fTimeStep;

		Clamp( m_fPosX, -g_BoardSizeX + g_FoodSize, g_BoardSizeX - g_FoodSize );
		Clamp( m_fPosY, -g_BoardSizeY + g_FoodSize, g_BoardSizeY - g_FoodSize );
	}

private:
	float m_fPosX;
	float m_fPosY;
	float m_fSpeedX;
	float m_fSpeedY;

};

Player g_Player[ 2 ];
std::vector< PacMan > g_PacMan;
std::vector< Food > g_Food;

float GetExtraCloneOffset()
{
	return (float)g_PacMan.size() * 1.0f;
}


void ResetGame()
{
	g_Player[ 0 ].Reset();
	g_Player[ 1 ].Reset();
	g_PacMan.resize( 0 );
	g_Food.resize( 0 );
}

void PrepareGame()
{
	if( g_iPlayerCount == 1 )
	{
		g_Player[ 0 ].SetPos( 0.0f, -g_BoardSizeY / 2.0f );
		g_Player[ 0 ].SetKeys( HGEK_SHIFT, HGEK_UP, HGEK_LEFT, HGEK_DOWN, HGEK_RIGHT );
	}
	else
	{
		g_Player[ 0 ].SetPos( 200.0f , -g_BoardSizeY / 2.0f );	
		g_Player[ 1 ].SetPos( -200.0f, -g_BoardSizeY / 2.0f );

		g_Player[ 0 ].SetKeys( HGEK_NUMPAD0, HGEK_UP, HGEK_LEFT, HGEK_DOWN, HGEK_RIGHT );
		g_Player[ 1 ].SetKeys( HGEK_G, HGEK_W, HGEK_A, HGEK_S, HGEK_D );
	}
	for( int i= 0; i < 11; ++i )
	{
		PacMan p;
		p.SetPos( (float)(i - 5 ) * g_BoardSizeX / 8.0f,
			g_BoardSizeY / 2.0f );
		g_PacMan.push_back( p );
	}

	g_GameOver= false;
	g_fGameTime= 0.0f;
	g_iScore= 0;
}

void PlayerPacManCollisions()
{
	float fSize= g_GhostSize + g_PacSize;
	for( int j= 0; j < g_iPlayerCount; ++j )
	{
		for( int i= g_PacMan.size() - 1 ; i >= 0; --i )
		{
			float fDistX= g_PacMan[ i ].GetX() - g_Player[ j ].GetX();
			float fDistY= g_PacMan[ i ].GetY() - g_Player[ j ].GetY();
			if( fDistX * fDistX + fDistY * fDistY < fSize * fSize )
			{
				if( g_Player[ j ].IsVulnerable() )
				{
					//tot -> game over
					g_GameOver= true;
					playFX( sndDead, g_PacMan[ i ].GetX() );
				}
				else
				{
					//fressen
					playFX( sndPacEaten, g_PacMan[ i ].GetX() );
					g_PacMan.erase( g_PacMan.begin() + i );
					g_Player[ j ].EatPacMan();
					g_iScore+= 100;
				}
			}
		}
	}

	if( g_PacMan.size() == 0 )
	{
		// -> game over
		g_GameOver= true;
		playFX( sndDead, ( g_Player[ 0 ].GetX() + g_Player[ 1 ].GetX() ) * 0.5f );
	}
}

void PacManFoodCollisions()
{
	float fSize= g_FoodSize + g_PacSize;
	for( int i= g_PacMan.size() - 1 ; i >= 0; --i )
	{
		for( int j= g_Food.size() - 1 ; j >= 0; --j )
		{
			float fDistX= g_PacMan[ i ].GetX() - g_Food[ j ].GetX();
			float fDistY= g_PacMan[ i ].GetY() - g_Food[ j ].GetY();
			if( fDistX * fDistX + fDistY * fDistY < fSize * fSize )
			{
				{
					//fressen
					playFX( sndFoodEaten, g_PacMan[ i ].GetX() );
					g_Food.erase( g_Food.begin() + j );
					g_PacMan[ i ].eat();
					if( g_iScore > 0 )
					{
						g_iScore-= 1;
					}
				}
			}
		}
	}
}

bool FrameFunc()
{
	float dt=hge->Timer_GetDelta();

	// Process keys
	if (hge->Input_GetKeyState(HGEK_ESCAPE)) return true;


	std::vector< PacMan > NewPacMan;
	for( int i= 0; i < g_PacMan.size(); ++i )
	{
		g_PacMan[ i ].calculate( dt );
		g_PacMan[ i ].canclone( NewPacMan );
	}

	if (hge->Input_GetKeyState(HGEK_F1) )
	{
		g_iPlayerCount= 1;
		ResetGame();
		PrepareGame();
	}
	if (hge->Input_GetKeyState(HGEK_F2) )
	{
		g_iPlayerCount= 2;
		ResetGame();
		PrepareGame();
	}
	if( g_GameOver )
	{
		if (hge->Input_GetKeyState(HGEK_R) || hge->Input_GetKeyState(HGEK_SPACE))
		{
			ResetGame();
			PrepareGame();
		}
	}
	else
	{
		g_fGameTime+= dt;
		g_PacMan.insert( g_PacMan.end(), NewPacMan.begin(), NewPacMan.end() );
		for( int i= 0; i < g_iPlayerCount; ++i )
		{
			g_Player[ i ].calculate( dt );
		}

		PlayerPacManCollisions();
		PacManFoodCollisions();

		for( int i= 0; i < g_Food.size(); ++i )
		{
			g_Food[ i ].calculate( dt, g_Food, g_Player );
		}
	}

	//eigentlich Test
	if( hge->Random_Float(0.0f, 1.0f) > 0.9f + (float)g_Food.size() * 0.0002f )
	{
		Food f;
		f.SetPos( 
			hge->Random_Float(-400.0f, 400.0f),
			hge->Random_Float(-300.0f, 300.0f) );

		g_Food.push_back( f );
	}

	return false;
}


bool RenderFunc()
{
	// Render graphics
	hge->Gfx_BeginScene();
	hge->Gfx_Clear(0);
	for( int i= 0; i < g_Food.size(); ++i )
	{
		g_Food[ i ].draw();
	}
	for( int i= 0; i < g_PacMan.size(); ++i )
	{
		g_PacMan[ i ].draw();
	}
	for( int i= 0; i < g_iPlayerCount; ++i )
	{
		g_Player[ i ].draw();
	}
	

	if( g_GameOver )
	{
		fnt->printf( 
			g_BoardSizeX, 
			g_BoardSizeY, 
			HGETEXT_CENTER,
			"Game Over!\nRestart with R or SPACE");
	}
	else if( g_fGameTime < 3.0f )
	{
		fnt->printf( 
			g_BoardSizeX, 
			40.0f, 
			HGETEXT_CENTER,
			"F1 - one player mode\nF2 - coop mode");

		if( g_iPlayerCount == 1 )
		{
			fnt->printf( 
				g_BoardSizeX, 
				g_BoardSizeY, 
				HGETEXT_CENTER,
				"move with CURSOR\nFocus with SHIFT");
		}
		else
		{
			fnt->printf( 
				g_BoardSizeX + 200.0f, 
				g_BoardSizeY, 
				HGETEXT_CENTER,
				"move with CURSOR\nFocus with Numpad 0");
			fnt->printf( 
				g_BoardSizeX - 200.0f, 
				g_BoardSizeY, 
				HGETEXT_CENTER,
				"move with WASD\nFocus with G");
		}
		fnt->printf( 
			g_BoardSizeX, 
			g_BoardSizeY + 280.0f, 
			HGETEXT_CENTER,
			"code: TGGC\nsoundtrack: Chrome - Stardust");
	}
		fnt->printf( 
			5, 
			5, 
			HGETEXT_LEFT,
			"Score: %07d", 
			g_iScore );
	hge->Gfx_EndScene();

	return false;
}


int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
{
	g_iPlayerCount= 1;
	hge = hgeCreate(HGE_VERSION);

	hge->Random_Seed( (int)time( NULL ));

	hge->System_SetState(HGE_LOGFILE, "error.log");
	hge->System_SetState(HGE_FRAMEFUNC, FrameFunc);
	hge->System_SetState(HGE_RENDERFUNC, RenderFunc);
	hge->System_SetState(HGE_TITLE, "Pacman Fast Prototype");
	hge->System_SetState(HGE_FPS, 100);
	hge->System_SetState(HGE_WINDOWED, true);
	hge->System_SetState(HGE_SCREENWIDTH, g_BoardSizeX * 2);
	hge->System_SetState(HGE_SCREENHEIGHT, g_BoardSizeY * 2);
	hge->System_SetState(HGE_SCREENBPP, 32);
	hge->System_SetState(HGE_SHOWSPLASH, false );

	if(hge->System_Initiate()) {

		// Load sound and texture
		sndEnergy=hge->Effect_Load("data/energy.ogg");
		sndPacEaten=hge->Effect_Load("data/paceaten.wav");
		sndFoodEaten=hge->Effect_Load("data/food.wav");
		sndDead=hge->Effect_Load("data/dead.wav");

		musGame = hge->Music_Load("data/stardust.xm");

		// Create and set up a sprites
		texGhost=hge->Texture_Load("data/ghost.png");
		sprGhost=new hgeSprite(texGhost, 0, 0, 32, 32);
		sprGhost->SetColor(0xFFFF0000);
		sprGhost->SetHotSpot(16,16);
		texGhost2=hge->Texture_Load("data/ghost_energy.png");
		sprGhost2=new hgeSprite(texGhost2, 0, 0, 48, 48);
		sprGhost2->SetColor(0xFFFF0000);
		sprGhost2->SetHotSpot(24,24);

		texPacMan=hge->Texture_Load("data/pac.png");
		sprPacMan=new hgeSprite(texPacMan, 0, 0, 16, 16);
		sprPacMan->SetColor(0xFFFFFF00);
		sprPacMan->SetHotSpot(8,8);

		texFood=hge->Texture_Load("data/food.png");
		sprFood=new hgeSprite(texFood, 0, 0, 6, 4);
		sprFood->SetColor(0xFF00A000);
		sprFood->SetHotSpot(3,2);

		// Load a font
		fnt=new hgeFont("data/font1.fnt");

		PrepareGame();

		// Let's rock now!
		hge->Music_Play(musGame, true);
		hge->System_Start();

		// Delete created objects and free loaded resources
		//bla...
	}

	// Clean up and shutdown
	hge->System_Shutdown();
	hge->Release();
	return 0;
}
